@@ -1,6 +1,8 @@
 import sys
+from argparse import Namespace
+from typing import Literal, Tuple, Union
 
-import waybackpy
+import waybackpy  # type: ignore
 
 
 class SavepagenowFailureError(Exception):
@@ -8,23 +10,29 @@ class SavepagenowFailureError(Exception):
 
 
 class Archiver:
-    def __init__(self, args):
-        """Init."""
-        self.retry = args.retry
-        self.UA = "Mozilla/5.0 (Windows NT 5.1; rv:40.0) " "Gecko/20100101 Firefox/40.0"
+    @staticmethod
+    def from_parser_args(args: Namespace) -> "Archiver":
+        return Archiver(args.retry)
 
-    def archive(self, url):
+    def __init__(self, retry: int):
+        """Init."""
+        self.retry: int = retry
+        self.UA: str = (
+            "Mozilla/5.0 (Windows NT 5.1; rv:40.0) " "Gecko/20100101 Firefox/40.0"
+        )
+
+    def archive(self, url: str) -> Union[Literal[False], Tuple[str, bool]]:
         """Archive link."""
-        wp = waybackpy.Url(url, self.UA)
+        wp = waybackpy.WaybackMachineSaveAPI(url, self.UA)
         for _ in range(self.retry + 1):
             if not self._try_savepagenow(wp):
                 continue
             else:
-                return (wp.archive_url, wp.cached_save)
+                return wp.archive_url, wp.cached_save
         return False
 
     @staticmethod
-    def _try_savepagenow(wp):
+    def _try_savepagenow(wp: waybackpy.Url) -> bool:
         """Error handler for saving with savepagenow."""
         try:
             wp.save()
