@@ -3,17 +3,18 @@ import argparse
 import http.client as httplib
 import sys
 import textwrap
+from typing import Iterable, List, Optional, Set
 
 from wbsv import __version__
 from wbsv.archiver import Archiver
-from wbsv.crawler import Clawler
+from wbsv.crawler import Crawler
 
 
 class HttpConnectionNotFountError(Exception):
     pass
 
 
-def check_connectivity(url="www.google.com", timeout=3):
+def check_connectivity(url: str = "www.google.com", timeout: int = 3) -> bool:
     conn = httplib.HTTPConnection(url, timeout=timeout)
     try:
         conn.request("HEAD", "/")
@@ -24,19 +25,19 @@ def check_connectivity(url="www.google.com", timeout=3):
         return False
 
 
-def check_natural(v):
+def check_natural(v: str) -> int:
     if int(v) < 0:
         raise argparse.ArgumentTypeError("%s is an invalid natural number" % int(v))
     return int(v)
 
 
-def check_positive(v):
+def check_positive(v: str) -> int:
     if int(v) <= 0:
         raise argparse.ArgumentTypeError("%s is an invalid natural number" % int(v))
     return int(v)
 
 
-def parse_args(test=None):
+def parse_args(args_list: Optional[List[str]] = None) -> argparse.Namespace:
     """Parse arguments."""
     parser = argparse.ArgumentParser(
         prog="wbsv",
@@ -79,76 +80,83 @@ def parse_args(test=None):
         help="Set maximum recursion depth. (>0)",
     )
     parser.add_argument(
-        "-V", "--version", action="version", version="%(prog)s {}".format(__version__)
+        "-O",
+        "--own",
+        action="store_true",
+        help="Only URLs with the same domain as target",
     )
-    if test:
-        return parser.parse_args(test)
+    parser.add_argument(
+        "-V", "--version", action="version", version=f"%(prog)s {__version__}"
+    )
+    if args_list:
+        return parser.parse_args(args_list)
     else:
         return parser.parse_args()
 
 
-def usual(args):
+def wbsv(
+    urls: Iterable[str], own: bool, only_target: bool, level: int, retry: int
+) -> None:
     past, now, fail = 0, 0, 0
-    print("[+]Target: {}".format(args.url))
-    c = Clawler(args)
-    retrieved_links = set().union(*c.run_crawl())
-    len_links = len(retrieved_links)
-    print("[+]{} URI(s) found.".format(len_links))
-    a = Archiver(args)
+    print(f"[+]Target: {urls}")
+    c = Crawler.from_args(urls=urls, own=own, only_target=only_target, level=level)
+    retrieved_links_empty_set: Set[str] = set()  # required for mypy type checking
+    retrieved_links: Set[str] = retrieved_links_empty_set.union(*c.run_crawl())
+    len_links: int = len(retrieved_links)
+    len_len_links: int = len(str(len_links))
+    print(f"[+]{len_links} URI(s) found.")
+    a = Archiver(retry)
     for ind, link in enumerate(retrieved_links, 1):
-        print("[{:02d}/{}]: Wait...".format(ind, len_links), end="\r")
+        print(f"[{ind:0{len_len_links}d}/{len_links}]: Wait...", end="\r")
         archive = a.archive(link)
         if archive:
             archived_link, cached_flag = archive
-            inc = cache_or_now(ind, len_links, archived_link, cached_flag)
+            cache_or_now: str = "PAST" if cached_flag else "NOW"
+            print(
+                f"[{ind:0{len_len_links}d}/{len_links}]: <{cache_or_now}> {archived_link}"
+            )
+            inc = (1, 0) if cached_flag else (0, 1)
             past += inc[0]
             now += inc[1]
         else:
-            print("[{:02d}/{}]: <FAIL> {}".format(ind, len_links, link))
+            print(f"[{ind:0{len_len_links}d}/{len_links}]: <FAIL> {link}")
             fail += 1
 
-    print("[+]FIN!: {}".format(args.url))
-    print("[+]ALL: {}, NOW: {}, PAST: {}, FAIL: {}".format(len_links, now, past, fail))
+    print(f"[+]FIN!: {urls}")
+    print(f"[+]ALL: {len_links}, NOW: {now}, PAST: {past}, FAIL: {fail}")
 
 
-def cache_or_now(ind, len_links, archived_link, cached_flag):
-    if cached_flag:
-        print("[{:02d}/{}]: <PAST> {}".format(ind, len_links, archived_link))
-        return (1, 0)
-    else:
-        print("[{:02d}/{}]: <NOW> {}".format(ind, len_links, archived_link))
-        return (0, 1)
+def wbsv_from_parser_args(args: argparse.Namespace) -> None:
+    wbsv(args.url, args.own, args.only_target, args.level, args.retry)
 
 
-def repl(args):
-    finish_words = ["end", "exit", "exit()", "break", "bye", ":q", "finish"]
+def wbsv_repl(args: argparse.Namespace) -> None:
+    finish_words = ("end", "exit", "exit()", "break", "bye", ":q", "finish")
     print("[[Input a target url (ex: https://google.com)]]")
     while True:
         link = input(">>> ").rstrip()
         if link in finish_words:
             print("[+]End.")
             break
-        elif link == "":
-            pass
-        else:
+        elif link != "":
             args.url = [link]
             try:
-                usual(args)
+                wbsv_from_parser_args(args)
             except Exception as e:
                 print(e, file=sys.stderr)
 
 
-def _main():
+def _main() -> None:
     if not check_connectivity():
         raise HttpConnectionNotFountError
     args = parse_args()
     if len(sys.argv) <= 1:
-        repl(args)
+        wbsv_repl(args)
     else:
-        usual(args)
+        wbsv_from_parser_args(args)
 
 
-def main():
+def main() -> None:
     try:
         _main()
     except KeyboardInterrupt:
